import os
import torch
import torch.nn as nn
from downloads_helper import check_download_inception

class InceptionV3W(nn.Module):
    """
    Wrapper around Inception V3 torchscript model provided here
    https://nvlabs-fi-cdn.nvidia.com/stylegan2-ada-pytorch/pretrained/metrics/inception-2015-12-05.pt
    
    path: locally saved inception weights
    """
    def __init__(self, path, download=True):
        super(InceptionV3W, self).__init__()
        # download the network if it is not present at the given directory
        # use the current directory by default
        if download: check_download_inception(fpath=path)
        path = os.path.join(path,"inception-2015-12-05.pt")
        self.base = torch.jit.load(path).eval()
        self.layers = self.base.layers
    
    """
    Get the inception features without resizing
    x: Image with values in range [0,255]
    """
    def forward(self, x):
        bs = x.shape[0]
        # make sure it is resized already
        assert x.shape[2]==299
        # apply normalization
        x1 = torch.sub(x, 128, 1)
        x2 = torch.div(x1, 128)
        features = self.layers.forward(x2, ).view((bs, 2048))
        return features